/*
 * c_can.h
 *
 *  Created on: Jul 19, 2011
 *      Author: csmith
 */

#ifndef C_CAN_H_
#define C_CAN_H_

#include "memtypes470M.h"
#include "macros.h"

#define MSG_RAM_LOCATION 0xFF1E0000 // each mailbox is 0x20 offset

enum MSG_DIR { // these states match bit in IF registers
	MSG_DIR_RECEIVE = 0,
	MSG_DIR_TRANSMIT = 1
};

enum MSG_STATE {
	MSG_STATE_EMPTY = 0, // read or transmitted
	MSG_STATE_READY = 1 // queued and ready for read or transmit
};

enum MSG_ID_TYPE {
	MSG_ID_TYPE_STD = 0, // 11-bit ID
	MSG_ID_TYPE_XTD = 1  // 29-bit ID
};

struct CAN_MESSAGE{
    INT8U dlc;       /* Data Length Code */
    INT32U id;       /* Message ID, if greater than 0x7FF, assumed extended ID*/
    INT8U data[8];   /* Message (maximal 8 bytes of data) */
    enum MSG_DIR dir;
    enum MSG_STATE state;
};

#define CAN1_IF1_BUSY (canREG1->IF1STAT & 0x80)
#define CAN1_IF2_BUSY (canREG1->IF2STAT & 0x80)

// used for IFxCMD
#define IF_CMD_WR	 		0x80 // this is a write to message ram
#define IF_CMD_RD			0x00 // this is a read from message ram
#define IF_UPDATE_MASK 		0x40
#define IF_UPDATE_ARB 		0x20
#define IF_UPDATE_CONTROL 	0x10
#define IF_UPDATE_CLR 		0x08
#define IF_UPDATE_TXRQST 	0x04
#define IF_UPDATE_DATAA 		0x02
#define IF_UPDATE_DATAB 		0x01
#define IF_UPDATE_ALL		0x7F  // this also clear new dat or requests tx

#define IF_MCTL_NEW_DAT		0x8000
#define IF_MCTL_MSG_LST		0x4000
#define IF_MCTL_INT_PND		0x2000
#define IF_MCTL_UMASK		0x1000
#define IF_MCTL_TXIE		0x0800
#define IF_MCTL_RXIE		0x0400
#define IF_MCTL_RMT_EN		0x0200
#define IF_MCTL_TX_RQST		0x0100
#define IF_MCTL_EOB			0x0080
#define IF_MCTL_DLC			0x000F

#define IF_ARB_MSG_VAL 		0x80000000
#define IF_ARB_XTD			0x40000000
#define IF_ARB_DIR_TX		0x20000000
#define IF_ARB_DIR_RX		0x00000000
#define IF_ARB_ID			0x1FFFFFFF

#define IF_MSK_MXTD			0x80000000
#define IF_MSK_MDIR			0x40000000

// todo: remove?
#define IF1_DATA_DIR_TX		(IF_ARB_DIR_TX & (canREG1->IF1ARB))
#define IF1_MSG_LOST		(IF_MCTL_MSG_LST & (canREG1->IF1MCTL))
#define IF1_NEW_DAT			(IF_MCTL_NEW_DAT & (canREG1->IF1MCTL))
#define IF1_TX_WAITING		(IF_MCTL_TX_RQST & (canREG1->IF1MCTL))

enum CAN_MB_UPDATE_STATUS {
    SUCCESS = 0,
	NON_MATCHING_MB_DIR = 1,
    RECEIVE_OVERRUN = 2,
    TRANSMIT_OVERRUN = 3,
    NO_NEW_CAN_RX_DATA_AVAILABLE = 4, // receive under-run
    NO_NEW_CAN_TX_DATA_READY = 5
};

void ccanInit();
void can_msg_set(int mailbox, enum MSG_DIR, enum MSG_ID_TYPE id_type, int id, int mask);
void can_update_buffer(enum MSG_DIR update_action, struct CAN_MESSAGE *local_msgs);
BOOLEAN newRcvMsg(int num);
enum CAN_MB_UPDATE_STATUS update_rx_mb_num(int num, struct CAN_MESSAGE *local_msg);
enum CAN_MB_UPDATE_STATUS update_tx_mb_num(int num, struct CAN_MESSAGE *local_msg);

#endif /* C_CAN_H_ */
