/** @file lin.c 
*   @brief LIN Driver Inmplmentation File
*   @date 15.March.2011
*   @version 1.00.001
*
*   (c) Texas Instruments 2011, All rights reserved.
*/

#if 0

#include "lin.h"


/** @fn void linInit(void)
*   @brief Initializes the lin Driver
*
*   This function initializes the lin module.
*/
void linInit(void)
{
    /** @b intalise @b LIN1 */

    /** - Release from reset */
    linREG1->GCR0 = 1U;

    /** - Setup control register 1
    *     - Enable transmitter
    *     - Enable receiver
    *     - Stop when debug mode is entered
    *     - Disable Loopback mode
    *     - Mask filtering with ID-Byte
    *     - Use enhance checksum
    *     - Enable multi buffer mode
    *     - Disable automatic baudrate adjustment
    *     - Disable sleep mode
    *     - Keep state machine in software reset
    *     - Enter LIN mode
    *     - Set LIN module as master
    *     - Enable/Disable parity
    *     - Disable data length control in ID4 and ID5
    */
    linREG1->GCR1 = 0x03000C60U | 0U;

    /** - Setup maximum baud rate prescaler */
    linREG1->MBRSR = 3600U;

    /** - Setup baud rate prescaler */
    linREG1->BRSR = 249U;

    /** - Setup RX and TX reception masks */
    linREG1->MASK = (0x00U << 16U) | 0x00U;

    /** - Setup compare
    *     - Sync delimiter
    *     - Sync break extension
    */
    linREG1->COMP = ((1U - 1U) << 8U) | (13U - 13U);

    /** - Setup response length */
    linREG1->LENGTH = (8U - 1U);

    /** - Set LIN pins functional mode 
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG1->FUN = 4U | 2U | 0U;

    /** - Set LIN pins default output value
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG1->DOUT = 0U | 0U | 0U;

    /** - Set LIN pins output direction
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG1->DIR = 0U | 0U | 0U;

    /** - Set LIN pins open drain enable
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG1->ODR = 0U | 0U | 0U;

    /** - Set LIN pins pullup/pulldown enable
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG1->PD = 0U | 0U | 0U;

    /** - Set LIN pins pullup/pulldown select
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG1->PSL = 4U | 2U | 1U;

    /** - Set interrupt level 
    *     - Bit error level
    *     - Physical bus error level
    *     - Checksum error level
    *     - Inconsistent sync field error level
    *     - No response error level
    *     - Framing error level
    *     - Overrun error level
    *     - Parity error level
    *     - Identifier level
    *     - RX level
    *     - TX level
    *     - Timeout after 3 wakeup signals level
    *     - Timeout after wakeup signal level
    *     - Timeout level
    *     - Wakeup level
    *     - Break detect level
    */
    linREG1->SETINTLVL = 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U;

    /** - Set interrupt enable 
    *     - Enable/Disable bit error
    *     - Enable/Disable physical bus error level
    *     - Enable/Disable checksum error level
    *     - Enable/Disable inconsistent sync field error level
    *     - Enable/Disable no response error level
    *     - Enable/Disable framing error level
    *     - Enable/Disable overrun error level
    *     - Enable/Disable parity error level
    *     - Enable/Disable identifier level
    *     - Enable/Disable RX level
    *     - Enable/Disable TX level
    *     - Enable/Disable timeout after 3 wakeup signals level
    *     - Enable/Disable timeout after wakeup signal level
    *     - Enable/Disable timeout level
    *     - Enable/Disable wakeup level
    *     - Enable/Disable break detect level
    */
    linREG1->SETINT = 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U;

    /** - Finaly start LIN1 */
    linREG1->GCR1 |= 0x00000080U;

    /** @b intalise @b LIN2 */

    /** - Release from reset */
    linREG2->GCR0 = 1U;

    /** - Setup control register 1
    *     - Enable transmitter
    *     - Enable receiver
    *     - Stop when debug mode is entered
    *     - Disable Loopback mode
    *     - Mask filtering with ID-Byte
    *     - Use enhance checksum
    *     - Enable multi buffer mode
    *     - Disable automatic baudrate adjustment
    *     - Disable sleep mode
    *     - Keep state machine in software reset
    *     - Enter LIN mode
    *     - Set LIN module as master
    *     - Enable/Disable parity
    *     - Disable data length control in ID4 and ID5
    */
    linREG2->GCR1 = 0x03000C60U | 0U;

    /** - Setup maximum baud rate prescaler */
    linREG2->MBRSR = 3600U;

    /** - Setup baud rate prescaler */
    linREG2->BRSR = 249U;

    /** - Setup RX and TX reception masks */
    linREG2->MASK = (0x00U << 16U) | 0x00U;

    /** - Setup compare
    *     - Sync delimiter
    *     - Sync break extension
    */
    linREG2->COMP = ((1U - 1U) << 8U) | (13U - 13U);

    /** - Setup response length */
    linREG2->LENGTH = (8U - 1U);

    /** - Set LIN pins functional mode 
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG2->FUN = 4U | 2U | 0U;

    /** - Set LIN pins default output value
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG2->DOUT = 0U | 0U | 0U;

    /** - Set LIN pins output direction
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG2->DIR = 0U | 0U | 0U;

    /** - Set LIN pins open drain enable
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG2->ODR = 0U | 0U | 0U;

    /** - Set LIN pins pullup/pulldown enable
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG2->PD = 0U | 0U | 0U;

    /** - Set LIN pins pullup/pulldown select
    *     - TX
    *     - RX
    *     - CLK
    */
    linREG2->PSL = 4U | 2U | 1U;

    /** - Set interrupt level 
    *     - Bit error level
    *     - Physical bus error level
    *     - Checksum error level
    *     - Inconsistent sync field error level
    *     - No response error level
    *     - Framing error level
    *     - Overrun error level
    *     - Parity error level
    *     - Identifier level
    *     - RX level
    *     - TX level
    *     - Timeout after 3 wakeup signals level
    *     - Timeout after wakeup signal level
    *     - Timeout level
    *     - Wakeup level
    *     - Break detect level
    */
    linREG2->SETINTLVL = 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U
                       | 0x00000000U;

    /** - Set interrupt enable 
    *     - Enable/Disable bit error
    *     - Enable/Disable physical bus error level
    *     - Enable/Disable checksum error level
    *     - Enable/Disable inconsistent sync field error level
    *     - Enable/Disable no response error level
    *     - Enable/Disable framing error level
    *     - Enable/Disable overrun error level
    *     - Enable/Disable parity error level
    *     - Enable/Disable identifier level
    *     - Enable/Disable RX level
    *     - Enable/Disable TX level
    *     - Enable/Disable timeout after 3 wakeup signals level
    *     - Enable/Disable timeout after wakeup signal level
    *     - Enable/Disable timeout level
    *     - Enable/Disable wakeup level
    *     - Enable/Disable break detect level
    */
    linREG2->SETINT = 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U
                    | 0x00000000U;

    /** - Finaly start LIN1 */
    linREG2->GCR1 |= 0x00000080U;

    /**   @note This function has to be called before the driver can be used.\n
    *           This function has to be executed in priviledged mode.\n
    */
}


/** @fn void linSetFunctional(linBASE_t *lin, unsigned port)
*   @brief Change functional behavoiur of pins at runtime.
*   @param[in] lin   - lin module base address
*   @param[in] port  - Value to write to FUN register
*
*   Change the value of the PCFUN register at runtime, this allows to
*   dynaimcaly change the functionality of the LIN pins between functional
*   and GIO mode.
*/
void linSetFunctional(linBASE_t *lin, unsigned port)
{
    lin->FUN = port;
}


/** @fn void linSendHeader(linBASE_t *lin, unsigned char identifier)
*   @brief Send lin header.
*   @param[in] lin  - lin module base address
*   @param[in] identifier - lin header id
*
*   Send lin header including sync break field, sync field and identifier.
*/
void linSendHeader(linBASE_t *lin, unsigned char identifier)
{
    lin->IDBYTE = identifier;
}


/** @fn void linSendWakupSignal(linBASE_t *lin)
*   @brief Send lin wakeup signal.
*   @param[in] lin  - lin module base address
*
*   Send lin wakeup signal to terminate the sleep mode of any lin node connected to the BUS.
*/
void linSendWakupSignal(linBASE_t *lin)
{
    lin->TDx[0] = 0x80;
    lin->GCR2   = 0x00000100U;
}


/** @fn void linSoftwareReset(linBASE_t *lin)
*   @brief Perform sofware reset.
*   @param[in] lin  - lin module base address
*
*   Perform software reset of lin module.
*   This function will reset the lin state machine and clear all pending flags.
*   It is required to call this function after a wakeup signal has been sent.
*/
void linSoftwareReset(linBASE_t *lin)
{
    lin->GCR2 &= ~0x00000080U;
    lin->GCR2 |=  0x00000080U;
}

/** @fn unsigned linIsTxReady(linBASE_t *lin)
*   @brief Check if Tx buffer empty
*   @param[in] lin - lin module base address
*
*   @return The TX ready flag
*
*   Checks to see if the Tx buffer ready flag is set, returns
*   0 is flags not set otherwise will return the Tx flag itself.
*/
unsigned linIsTxReady(linBASE_t *lin)
{
    return lin->FLR & LIN_TX_READY;
}

/** @fn void linSetLength(linBASE_t *lin, unsigned length)
*   @brief Send Data
*   @param[in] lin    - lin module base address
*   @param[in] length - number of data words in bytes. Range: 1-8.
*
*   Send data response length in bytes. 
*/
void linSetLength(linBASE_t *lin, unsigned length)
{
    lin->LENGTH = length - 1U;
}

/** @fn void linSend(linBASE_t *lin, const unsigned char *data)
*   @brief Send Data
*   @param[in] lin    - lin module base address
*   @param[in] data   - pointer to data to send
*
*   Send a block of data pointed to by 'data'.
*   The number of data to transmit must be set with 'linSetLength' before. 
*/
void linSend(linBASE_t *lin, const unsigned char *data)
{
    int           i;
    int           length = lin->LENGTH;
    unsigned char *pData = (unsigned char*)data + length;

    for (i = length; i >= 0; i--)
    {
        lin->TDx[i] = *pData--;
    }
}

/** @fn unsigned linIsRxReady(linBASE_t *lin)
*   @brief Check if Rx buffer full
*   @param[in] lin - lin module base address
*
*   @return The Rx ready flag
*
*   Checks to see if the Rx buffer full flag is set, returns
*   0 is flags not set otherwise will return the Rx flag itself.
*/
unsigned linIsRxReady(linBASE_t *lin)
{
    return lin->FLR & LIN_RX_INT;
}


/** @fn unsigned linTxRxError(linBASE_t *lin)
*   @brief Return Tx and Rx Error flags
*   @param[in] lin - lin module base address
*
*   @return The Tx and Rx error flags
*
*   Returns the bit, physical bus, checksum, inconsisten sync field,
*   no response, framing, overun, parity and timeout error flags.
*   It also clears the error flags before returning.
*/
unsigned linTxRxError(linBASE_t *lin)
{
    unsigned status = lin->FLR & (LIN_BE_INT 
                    | LIN_PBE_INT 
                    | LIN_CE_INT 
                    | LIN_ISFE_INT 
                    | LIN_NRE_INT 
                    | LIN_FE_INT 
                    | LIN_OE_INT 
                    | LIN_PE_INT     
                    | LIN_TOA3WUS_INT 
                    | LIN_TOAWUS_INT 
                    | LIN_TO_INT);

    lin->FLR = LIN_BE_INT 
             | LIN_PBE_INT 
             | LIN_CE_INT 
             | LIN_ISFE_INT 
             | LIN_NRE_INT 
             | LIN_FE_INT 
             | LIN_OE_INT 
             | LIN_PE_INT     
             | LIN_TOA3WUS_INT 
             | LIN_TOAWUS_INT 
             | LIN_TO_INT;

    return status;
}


/** @fn unsigned linGetIdentifier(linBASE_t *lin)
*   @brief Get last received identifier
*   @param[in] lin - lin module base address
*
*   @return Identifier
*
*   Read last received identifier.
*/
unsigned linGetIdentifier(linBASE_t *lin)
{
    return lin->RXID;
}


/** @fn void linGetData(linBASE_t *lin, unsigned char * const data)
*   @brief Read received data
*   @param[in] lin    - lin module base address
*   @param[in] data   - pointer to data buffer
*
*   Read a block of bytes and place it into the data buffer pointed to by 'data'.
*/
void linGetData(linBASE_t *lin, unsigned char * const data)
{
    unsigned      i;
    unsigned      length = lin->LENGTH;
    unsigned char *pData = (unsigned char*)data;

    for (i = 0U; i <= length; i++)
    {
        *pData++ = lin->RDx[i];
    }
}


/** @fn linEnableNotification(linBASE_t *lin, unsigned flags)
*   @brief Enable interrupts
*   @param[in] lin   - lin module base address
*   @param[in] flags - Interrupts to be enabled, can be ored value of:
*                      LIN_BE_INT      - bit error,
*                      LIN_PBE_INT     - physical bus error,
*                      LIN_CE_INT      - checksum error,
*                      LIN_ISFE_INT    - inconsistent sync field error,
*                      LIN_NRE_INT     - no response error,
*                      LIN_FE_INT      - framming error,
*                      LIN_OE_INT      - overrun error,
*                      LIN_PE_INT      - parity error,
*                      LIN_ID_INT      - received matching identifier,
*                      LIN_RX_INT      - receive buffer ready,
*                      LIN_TOA3WUS_INT - time out after 3 wakeup signals,
*                      LIN_TOAWUS_INT  - time out after wakeup signal,
*                      LIN_TO_INT      - time out signal,
*                      LIN_WAKEUP_INT  - wakeup,
*                      LIN_BREAK_INT   - break detect
*/
void linEnableNotification(linBASE_t *lin, unsigned flags)
{
    lin->SETINT = flags;
}


/** @fn linDisableNotification(linBASE_t *lin, unsigned flags)
*   @brief Disable interrupts
*   @param[in] lin   - lin module base address
*   @param[in] flags - Interrupts to be disabled, can be ored value of:
*                      LIN_BE_INT      - bit error,
*                      LIN_PBE_INT     - physical bus error,
*                      LIN_CE_INT      - checksum error,
*                      LIN_ISFE_INT    - inconsistent sync field error,
*                      LIN_NRE_INT     - no response error,
*                      LIN_FE_INT      - framming error,
*                      LIN_OE_INT      - overrun error,
*                      LIN_PE_INT      - parity error,
*                      LIN_ID_INT      - received matching identifier,
*                      LIN_RX_INT      - receive buffer ready,
*                      LIN_TOA3WUS_INT - time out after 3 wakeup signals,
*                      LIN_TOAWUS_INT  - time out after wakeup signal,
*                      LIN_TO_INT      - time out signal,
*                      LIN_WAKEUP_INT  - wakeup,
*                      LIN_BREAK_INT   - break detect
*/
void linDisableNotification(linBASE_t *lin, unsigned flags)
{
    lin->CLRINT = flags;
}


/** @fn void sci1HighLevelInterrupt(void)
*   @brief High Level Interrupt for LIN1
*/
#if 1
    #pragma INTERRUPT(sci1HighLevelInterrupt)
#endif

void sci1HighLevelInterrupt(void)
{
    unsigned vec = linREG1->INTVECT0;

    switch (vec)
    {
    case 1:  linNotification(linREG1, LIN_WAKEUP_INT);  break;
    case 2:  linNotification(linREG1, LIN_ISFE_INT);    break;
    case 3:  linNotification(linREG1, LIN_PE_INT);      break;
    case 4:  linNotification(linREG1, LIN_ID_INT);      break;
    case 5:  linNotification(linREG1, LIN_PBE_INT);     break;
    case 6:  linNotification(linREG1, LIN_FE_INT);      break;
    case 7:  linNotification(linREG1, LIN_BREAK_INT);   break;
    case 8:  linNotification(linREG1, LIN_CE_INT);      break;
    case 9:  linNotification(linREG1, LIN_OE_INT);      break;
    case 10: linNotification(linREG1, LIN_BE_INT);      break;
    case 11: linNotification(linREG1, LIN_RX_INT);      break;
    case 13: linNotification(linREG1, LIN_NRE_INT);     break;
    case 14: linNotification(linREG1, LIN_TOAWUS_INT);  break;
    case 15: linNotification(linREG1, LIN_TOA3WUS_INT); break;
    case 16: linNotification(linREG1, LIN_TO_INT);      break;
    default:
        /* phantom interrupt, perform software reset */
        linREG1->GCR2 &= ~0x00000080U;
        linREG1->GCR2 |=  0x00000080U;
    }
}


/** @fn void sci1LowLevelInterrupt(void)
*   @brief Low Level Interrupt for LIN1
*/
#if 1
    #pragma INTERRUPT(sci1LowLevelInterrupt)
#endif

void sci1LowLevelInterrupt(void)
{
    unsigned vec = linREG1->INTVECT1;

    switch (vec)
    {
    case 1:  linNotification(linREG1, LIN_WAKEUP_INT);  break;
    case 2:  linNotification(linREG1, LIN_ISFE_INT);    break;
    case 3:  linNotification(linREG1, LIN_PE_INT);      break;
    case 4:  linNotification(linREG1, LIN_ID_INT);      break;
    case 5:  linNotification(linREG1, LIN_PBE_INT);     break;
    case 6:  linNotification(linREG1, LIN_FE_INT);      break;
    case 7:  linNotification(linREG1, LIN_BREAK_INT);   break;
    case 8:  linNotification(linREG1, LIN_CE_INT);      break;
    case 9:  linNotification(linREG1, LIN_OE_INT);      break;
    case 10: linNotification(linREG1, LIN_BE_INT);      break;
    case 11: linNotification(linREG1, LIN_RX_INT);      break;
    case 13: linNotification(linREG1, LIN_NRE_INT);     break;
    case 14: linNotification(linREG1, LIN_TOAWUS_INT);  break;
    case 15: linNotification(linREG1, LIN_TOA3WUS_INT); break;
    case 16: linNotification(linREG1, LIN_TO_INT);      break;
    default:
        /* phantom interrupt, perform software reset */
        linREG1->GCR2 &= ~0x00000080U;
        linREG1->GCR2 |=  0x00000080U;
    }
}


/** @fn void sci2HighLevelInterrupt(void)
*   @brief High Level Interrupt for LIN2
*/
#if 1
    #pragma INTERRUPT(sci2HighLevelInterrupt)
#endif

void sci2HighLevelInterrupt(void)
{
    unsigned vec = linREG2->INTVECT0;

    switch (vec)
    {
    case 1:  linNotification(linREG2, LIN_WAKEUP_INT);  break;
    case 2:  linNotification(linREG2, LIN_ISFE_INT);    break;
    case 3:  linNotification(linREG2, LIN_PE_INT);      break;
    case 4:  linNotification(linREG2, LIN_ID_INT);      break;
    case 5:  linNotification(linREG2, LIN_PBE_INT);     break;
    case 6:  linNotification(linREG2, LIN_FE_INT);      break;
    case 7:  linNotification(linREG2, LIN_BREAK_INT);   break;
    case 8:  linNotification(linREG2, LIN_CE_INT);      break;
    case 9:  linNotification(linREG2, LIN_OE_INT);      break;
    case 10: linNotification(linREG2, LIN_BE_INT);      break;
    case 11: linNotification(linREG2, LIN_RX_INT);      break;
    case 13: linNotification(linREG2, LIN_NRE_INT);     break;
    case 14: linNotification(linREG2, LIN_TOAWUS_INT);  break;
    case 15: linNotification(linREG2, LIN_TOA3WUS_INT); break;
    case 16: linNotification(linREG2, LIN_TO_INT);      break;
    default:
        /* phantom interrupt, perform software reset */
        linREG2->GCR2 &= ~0x00000080U;
        linREG2->GCR2 |=  0x00000080U;
    }
}


/** @fn void sci2LowLevelInterrupt(void)
*   @brief Low Level Interrupt for LIN2
*/
#if 1
    #pragma INTERRUPT(sci2LowLevelInterrupt)
#endif

void sci2LowLevelInterrupt(void)
{
    unsigned vec = linREG2->INTVECT1;

    switch (vec)
    {
    case 1:  linNotification(linREG2, LIN_WAKEUP_INT);  break;
    case 2:  linNotification(linREG2, LIN_ISFE_INT);    break;
    case 3:  linNotification(linREG2, LIN_PE_INT);      break;
    case 4:  linNotification(linREG2, LIN_ID_INT);      break;
    case 5:  linNotification(linREG2, LIN_PBE_INT);     break;
    case 6:  linNotification(linREG2, LIN_FE_INT);      break;
    case 7:  linNotification(linREG2, LIN_BREAK_INT);   break;
    case 8:  linNotification(linREG2, LIN_CE_INT);      break;
    case 9:  linNotification(linREG2, LIN_OE_INT);      break;
    case 10: linNotification(linREG2, LIN_BE_INT);      break;
    case 11: linNotification(linREG2, LIN_RX_INT);      break;
    case 13: linNotification(linREG2, LIN_NRE_INT);     break;
    case 14: linNotification(linREG2, LIN_TOAWUS_INT);  break;
    case 15: linNotification(linREG2, LIN_TOA3WUS_INT); break;
    case 16: linNotification(linREG2, LIN_TO_INT);      break;
    default:
        /* phantom interrupt, perform software reset */
        linREG2->GCR2 &= ~0x00000080U;
        linREG2->GCR2 |=  0x00000080U;
    }
}

#endif
