/** @file sci.c 
*   @brief SCI Driver Inmplmentation File
*   @date 15.March.2011
*   @version 1.00.000
*
*   (c) Texas Instruments 2011, All rights reserved.
*/

#if 1

#include "sci.h"


/** @struct g_sciTransfer
*   @brief Interrupt mode globals
*
*/
struct g_sciTransfer
{
    unsigned       mode;
    unsigned       length;
    unsigned char *data;
} g_sciTransfer[2];


/** @fn void sciInit(void)
*   @brief Initializes the SCI Driver
*
*   This function initializes the SCI module.
*/
void sciInit(void)
{
    /** @b intalise @b SCI1 */

    /** - bring SCI out of reset */
    sciREG1->GCR0 = 1U;

    /** - Disable all interrupts */
    sciREG1->CLRINT    = 0xFFFFFFFF;
    sciREG1->CLRINTLVL = 0xFFFFFFFF;

    /** - global control 1 */
    sciREG1->GCR1 = (1 << 25)  /* enable transmit */
                  | (1 << 24)  /* enable receive */
                  | (1 << 5)   /* internal clock (device has no clock pin) */
                  | ((1-1) << 4)  /* number of stop bits */
                  | (0 << 3)  /* even parity, otherwise odd */
                  | (0 << 2)  /* enable parity */
                  | (1 << 1);  /* asynchronous timing mode */

    /** - set baudrate */
    sciREG1->BAUD = 520;  /* baudrate */

    /** - tranmision length */
    sciREG1->LENGTH = 8 - 1;  /* length */

    /** - set SCI pins functional mode */
    sciREG1->FUN = (1 << 2)  /* tx pin */
                 | (1 << 1)  /* rx pin */
                 | (0);  /* clk pin */

    /** - set SCI pins default output value */
    sciREG1->DOUT = (0 << 2)  /* tx pin */
                  | (0 << 1)  /* rx pin */
                  | (0);  /* clk pin */

    /** - set SCI pins output direction */
    sciREG1->DIR = (1 << 2)  /* tx pin */
                 | (0 << 1)  /* rx pin */
                 | (0);  /* clk pin */

    /** - set SCI pins open drain enable */
    sciREG1->ODR = (0 << 2)  /* tx pin */
                 | (0 << 1)  /* rx pin */
                 | (0);  /* clk pin */

    /** - set SCI pins pullup/pulldown enable */
    sciREG1->PD = (0 << 2)  /* tx pin */
                | (0 << 1)  /* rx pin */
                | (0);  /* clk pin */

    /** - set SCI pins pullup/pulldown select */
    sciREG1->PSL = (1 << 2)  /* tx pin */
                 | (1 << 1)  /* rx pin */
                 | (1);  /* clk pin */

    /** - set interrupt level */
    sciREG1->SETINTLVL = (0 << 26)  /* Framing error */
                       | (0 << 25)  /* Overrun error */
                       | (0 << 24)  /* Pariry error */
                       | (0 << 9)  /* Receive */
                       | (0 << 8)  /* Transmit */
                       | (0 << 1)  /* Wakeup */
                       | (0);  /* Break detect */

    /** - set interrupt enable */
    sciREG1->SETINT = (0 << 26)  /* Framing error */
                    | (0 << 25)  /* Overrun error */
                    | (0 << 24)  /* Pariry error */
                    | (0 << 9)  /* Receive */
                    | (0 << 1)  /* Wakeup */
                    | (0);  /* Break detect */

    /** - inialise global transfer variables */
    g_sciTransfer[0].mode   = 0 << 8;
    g_sciTransfer[0].length = 0;

    /** - Finaly start SCI1 */
    sciREG1->GCR1 |= (1 << 7);

    /** @b intalise @b SCI2 */

    /** - bring SCI out of reset */
    sciREG2->GCR0 = 1U;

    /** - Disable all interrupts */
    sciREG2->CLRINT    = 0xFFFFFFFF;
    sciREG2->CLRINTLVL = 0xFFFFFFFF;

    /** - global control 1 */
    sciREG2->GCR1 = (1 << 25)  /* enable transmit */
                  | (1 << 24)  /* enable receive */
                  | (1 << 5)   /* internal clock (device has no clock pin) */
                  | ((2-1) << 4)  /* number of stop bits */
                  | (0 << 3)  /* even parity, otherwise odd */
                  | (0 << 2)  /* enable parity */
                  | (1 << 1);  /* asynchronous timing mode */

    /** - set baudrate */
    sciREG2->BAUD = 520;  /* baudrate */

    /** - tranmision length */
    sciREG2->LENGTH = 8 - 1;  /* length */

    /** - set SCI pins functional mode */
    sciREG2->FUN = (1 << 2)  /* tx pin */
                 | (1 << 1)  /* rx pin */
                 | (0);  /* clk pin */

    /** - set SCI pins default output value */
    sciREG2->DOUT = (0 << 2)  /* tx pin */
                  | (0 << 1)  /* rx pin */
                  | (0);  /* clk pin */

    /** - set SCI pins output direction */
    sciREG2->DIR = (0 << 2)  /* tx pin */
                 | (0 << 1)  /* rx pin */
                 | (0);  /* clk pin */

    /** - set SCI pins open drain enable */
    sciREG2->ODR = (0 << 2)  /* tx pin */
                 | (0 << 1)  /* rx pin */
                 | (0);  /* clk pin */

    /** - set SCI pins pullup/pulldown enable */
    sciREG2->PD = (0 << 2)  /* tx pin */
                | (0 << 1)  /* rx pin */
                | (0);  /* clk pin */

    /** - set SCI pins pullup/pulldown select */
    sciREG2->PSL = (1 << 2)  /* tx pin */
                 | (1 << 1)  /* rx pin */
                 | (1);  /* clk pin */

    /** - set interrupt level */
    sciREG2->SETINTLVL = (0 << 26)  /* Framing error */
                       | (0 << 25)  /* Overrun error */
                       | (0 << 24)  /* Pariry error */
                       | (0 << 9)  /* Receive */
                       | (0 << 8)  /* Transmit */
                       | (0 << 1)  /* Wakeup */
                       | (0);  /* Break detect */

    /** - set interrupt enable */
    sciREG2->SETINT = (0 << 26)  /* Framing error */
                    | (0 << 25)  /* Overrun error */
                    | (0 << 24)  /* Pariry error */
                    | (0 << 9)  /* Receive */
                    | (0 << 1)  /* Wakeup */
                    | (0);  /* Break detect */

    /** - inialise global transfer variables */
    g_sciTransfer[1].mode   = 0 << 8;
    g_sciTransfer[1].length = 0;

    /** - Finaly start SCI2 */
    sciREG2->GCR1 |= (1 << 7);
}


/** @fn void sciSetFunctional(sciBASE_t *sci, unsigned port)
*   @brief Change functional behavoiur of pins at runtime.
*   @param[in] sci   - sci module base address
*   @param[in] port  - Value to write to FUN register
*
*   Change the value of the PCFUN register at runtime, this allows to
*   dynaimcaly change the functionality of the SCI pins between functional
*   and GIO mode.
*/
void sciSetFunctional(sciBASE_t *sci, unsigned port)
{
    sci->FUN = port;
}


/** @fn void sciSetBaudrate(sciBASE_t *sci, unsigned baud)
*   @brief Change baudrate at runtime.
*   @param[in] sci  - sci module base address
*   @param[in] baud - baudrate in Hz
*
*   Change the SCI baudrate at runtime.
*/
void sciSetBaudrate(sciBASE_t *sci, unsigned baud)
{
    double   vclk = 80.000 * 1000000.0;
    unsigned f    = sci->GCR1 & 2 ? 16 : 1;

    sci->BAUD = ((unsigned)((vclk /(f*baud) + 0.5)) - 1) & 0x00FFFFFF;
}


/** @fn int sciIsTxReady(sciBASE_t *sci)
*   @brief Check if Tx buffer empty
*   @param[in] sci - sci module base address
*
*   @return The TX ready flag
*
*   Checks to see if the Tx buffer ready flag is set, returns
*   0 is flags not set otherwise will return the Tx flag itself.
*/
int sciIsTxReady(sciBASE_t *sci)
{
    return sci->FLR & SCI_TX_INT;
}


/** @fn void sciSendByte(sciBASE_t *sci, unsigned char byte)
*   @brief Send Byte
*   @param[in] sci  - sci module base address
*   @param[in] byte - byte to transfer
*
*   Sends a single byte in polling mode, will wait in the
*   routine until the transmit buffer is empty before sending
*   the byte.  Use sciIsTxReady to check for Tx buffer empty
*   before calling sciSendByte to avoid waiting.
*/
void sciSendByte(sciBASE_t *sci, unsigned char byte)
{
    while ((sci->FLR & SCI_TX_INT) == 0) { /* wait */ };
    sci->TD = byte;
}


/** @fn void sciSend(sciBASE_t *sci, unsigned length, unsigned char *data)
*   @brief Send Data
*   @param[in] sci    - sci module base address
*   @param[in] length - number of data words to transfer
*   @param[in] data   - pointer to data to send
*
*   Send a block of data pointed to by 'data' and 'length' bytes
*   long.  If interrupts have been enabled the data is sent using
*   interrupt mode, otherwise polling mode is used.  In interrupt
*   mode transmition of the first byte is started and the routine
*   returns imediatly, sciSend must not be called again until the
*   transfer is complete, when the sciNotification callback will
*   be called.  In polling mode, sciSend will not return  until 
*   the transfer is complete.
*
*   @note if data word is less than 8 bits, then the data must be left
*         aligned in the data byte.
*/
void sciSend(sciBASE_t *sci, unsigned length, unsigned char *data)
{
    int index = sci == sciREG1 ? 0 : 1;

    if (g_sciTransfer[index].mode & SCI_TX_INT)
    {
        /* we are in interrupt mode */
        
        g_sciTransfer[index].length = length;
        g_sciTransfer[index].data   = data;

        /* start transmit by sending first byte */        
        sci->TD     = *g_sciTransfer[index].data++;
        sci->SETINT = SCI_TX_INT;
    }
    else
    {
        /* send the data */
        while (length-- > 0)
        {
            while ((sci->FLR & SCI_TX_INT) == 0) { /* wait */ };
            sci->TD = *data++;
        }
    }
}


/** @fn int sciIsRxReady(sciBASE_t *sci)
*   @brief Check if Rx buffer full
*   @param[in] sci - sci module base address
*
*   @return The Rx ready flag
*
*   Checks to see if the Rx buffer full flag is set, returns
*   0 is flags not set otherwise will return the Rx flag itself.
*/
int sciIsRxReady(sciBASE_t *sci)
{
    return sci->FLR & SCI_RX_INT;
}


/** @fn int sciRxError(sciBASE_t *sci)
*   @brief Return Rx Error flags
*   @param[in] sci - sci module base address
*
*   @return The Rx error flags
*
*   Returns the Rx framing, overun and parity errors flags,
*   also clears the error flags before returning.
*/
int sciRxError(sciBASE_t *sci)
{
    int status = sci->FLR & (SCI_FE_INT | SCI_OE_INT |SCI_PE_INT);

    sci->FLR = SCI_FE_INT | SCI_OE_INT | SCI_PE_INT;

    return status;
}


/** @fn unsigned sciReceiveByte(sciBASE_t *sci)
*   @brief Receive Byte
*   @param[in] sci - sci module base address
*
*   @return Received byte
*
*    Recieves a single byte in polling mode.  If there is
*    not a byte in the receive buffer the routine will wait
*    until one is received.   Use sciIsRxReady to check to
*    see if the buffer is full to avoid waiting.
*/
int sciReceiveByte(sciBASE_t *sci)
{
    while ((sci->FLR & SCI_RX_INT) == 0) { /* wait */ };

    return sci->RD;
}


/** @fn void sciReceive(sciBASE_t *sci, unsigned length, unsigned char *data)
*   @brief Receive Data
*   @param[in] sci    - sci module base address
*   @param[in] length - number of data words to transfer
*   @param[in] data   - pointer to data buffer
*
*   Receive a block of 'length' bytes long and place it into the 
*   data buffer pointed to by 'data'.  If interrupts have been 
*   enabled the data is received using interrupt mode, otherwise
*   polling mode is used.  In interrupt mode receive is setup and
*   the routine returns imediatly, sciReceive must not be called 
*   again until the transfer is complete, when the sciNotification 
*   callback will be called.  In polling mode, sciReceive will not
*   return  until the transfer is complete.
*/
void sciReceive(sciBASE_t *sci, unsigned length, unsigned char *data)
{
    if (sci->SETINT & SCI_RX_INT)
    {
        /* we are in interrupt mode */
        int index = sci == sciREG1 ? 0 : 1;
        
        /* clear error flags */
        sci->FLR = SCI_FE_INT | SCI_OE_INT | SCI_PE_INT;

        g_sciTransfer[index].length = length;
        g_sciTransfer[index].data   = data;
    }
    else
    {   
        while (length-- > 0)
        {
            while ((sci->FLR & SCI_RX_INT) == 0) { /* wait */ };
            *data++ = sci->RD;
        }
    }
}


/** @fn sciEnableNotification(sciBASE_t *sci, unsigned flags)
*   @brief Enable interrupts
*   @param[in] sci   - sci module base address
*   @param[in] flags - Interrupts to be enabled, can be ored value of:
*                      SCI_FE_INT    - framming error,
*                      SCI_OE_INT    - overrun error,
*                      SCI_PE_INT    - parity error,
*                      SCI_RX_INT    - receive buffer ready,
*                      SCI_TX_INT    - transmit buffer ready,
*                      SCI_WAKE_INT  - wakeup,
*                      SCI_BREAK_INT - break detect
*/
void sciEnableNotification(sciBASE_t *sci, unsigned flags)
{
    int index = sci == sciREG1 ? 0 : 1;

    g_sciTransfer[index].mode |= (flags & SCI_TX_INT);
    sci->SETINT                = (flags & ~SCI_TX_INT);
}


/** @fn sciDisableNotification(sciBASE_t *sci, unsigned flags)
*   @brief Disable interrupts
*   @param[in] sci   - sci module base address
*   @param[in] flags - Interrupts to be disabled, can be ored value of:
*                      SCI_FE_INT    - framming error,
*                      SCI_OE_INT    - overrun error,
*                      SCI_PE_INT    - parity error,
*                      SCI_RX_INT    - receive buffer ready,
*                      SCI_TX_INT    - transmit buffer ready,
*                      SCI_WAKE_INT  - wakeup,
*                      SCI_BREAK_INT - break detect
*/
void sciDisableNotification(sciBASE_t *sci, unsigned flags)
{
    int index = sci == sciREG1 ? 0 : 1;

    g_sciTransfer[index].mode &= ~(flags & SCI_TX_INT);
    sci->CLRINT                = (flags & ~SCI_TX_INT);
}


/** @fn void sci1HighLevelInterrupt(void)
*   @brief High Level Interrupt for SCI1
*/
#if 1
    #pragma INTERRUPT(sci1HighLevelInterrupt)
#endif

void sci1HighLevelInterrupt(void)
{
    unsigned vec = sciREG1->INTVECT0;

    switch (vec)
    {
    case 1:
        sciNotification(sciREG1, SCI_WAKE_INT);
        break;
    case 3:
        sciNotification(sciREG1, SCI_PE_INT);
        break;
    case 6:
        sciNotification(sciREG1, SCI_FE_INT);
        break;
    case 7:
        sciNotification(sciREG1, SCI_BREAK_INT);
        break;
    case 9:
        sciNotification(sciREG1, SCI_OE_INT);
        break;

    case 11:
        /* receive */
        {   unsigned byte = sciREG1->RD;

            if (g_sciTransfer[0].length > 0)
            {
                *g_sciTransfer[0].data++ = byte;
                g_sciTransfer[0].length--;
                if (g_sciTransfer[0].length == 0)
                {
                    sciNotification(sciREG1, SCI_RX_INT);
                }
            }
        }
        break;

    case 12:
        /* transmit */
        if (--g_sciTransfer[0].length > 0)
        {
            sciREG1->TD = *g_sciTransfer[0].data++;
        }
        else
        {
            sciREG1->CLRINT = SCI_TX_INT;
            sciNotification(sciREG1, SCI_TX_INT);
        }
        break;

    default:
        /* phantom interrupt, clear flags and return */
        sciREG1->FLR = ~sciREG1->SETINTLVL & 0x07000303;
    }
}


/** @fn void sci1LowLevelInterrupt(void)
*   @brief Low Level Interrupt for SCI1
*/
#if 1
    #pragma INTERRUPT(sci1LowLevelInterrupt)
#endif

void sci1LowLevelInterrupt(void)
{
    unsigned vec = sciREG1->INTVECT1;

    switch (vec)
    {
    case 1:
        sciNotification(sciREG1, SCI_WAKE_INT);
        break;
    case 3:
        sciNotification(sciREG1, SCI_PE_INT);
        break;
    case 6:
        sciNotification(sciREG1, SCI_FE_INT);
        break;
    case 7:
        sciNotification(sciREG1, SCI_BREAK_INT);
        break;
    case 9:
        sciNotification(sciREG1, SCI_OE_INT);
        break;

    case 11:
        /* receive */
        {   unsigned byte = sciREG1->RD;

            if (g_sciTransfer[0].length > 0)
            {
                *g_sciTransfer[0].data++ = byte;
                g_sciTransfer[0].length--;
                if (g_sciTransfer[0].length == 0)
                {
                    sciNotification(sciREG1, SCI_RX_INT);
                }
            }
        }
        break;

    case 12:
        /* transmit */
        if (--g_sciTransfer[0].length > 0)
        {
            sciREG1->TD = *g_sciTransfer[0].data++;
        }
        else
        {
            sciREG1->CLRINT = SCI_TX_INT;
            sciNotification(sciREG1, SCI_TX_INT);
        }
        break;

    default:
        /* phantom interrupt, clear flags and return */
        sciREG1->FLR = sciREG1->SETINTLVL & 0x07000303;
    }
}


/** @fn void sci2HighLevelInterrupt(void)
*   @brief High Level Interrupt for SCI2
*/
#if 1
    #pragma INTERRUPT(sci2HighLevelInterrupt)
#endif

void sci2HighLevelInterrupt(void)
{
    unsigned vec = sciREG2->INTVECT0;

    switch (vec)
    {
    case 1:
        sciNotification(sciREG2, SCI_WAKE_INT);
        break;
    case 3:
        sciNotification(sciREG2, SCI_PE_INT);
        break;
    case 6:
        sciNotification(sciREG2, SCI_FE_INT);
        break;
    case 7:
        sciNotification(sciREG2, SCI_BREAK_INT);
        break;
    case 9:
        sciNotification(sciREG2, SCI_OE_INT);
        break;

    case 11:
        /* receive */
        {   unsigned byte = sciREG2->RD;

            if (g_sciTransfer[1].length > 0)
            {
                *g_sciTransfer[1].data++ = byte;
                g_sciTransfer[1].length--;
                if (g_sciTransfer[1].length == 0)
                {
                    sciNotification(sciREG2, SCI_RX_INT);
                }
            }
        }
        break;

    case 12:
        /* transmit */
        if (--g_sciTransfer[1].length > 0)
        {
            sciREG2->TD = *g_sciTransfer[0].data++;
        }
        else
        {
            sciREG2->CLRINT = SCI_TX_INT;
            sciNotification(sciREG2, SCI_TX_INT);
        }
        break;

    default:
        /* phantom interrupt, clear flags and return */
        sciREG2->FLR = ~sciREG2->SETINTLVL & 0x07000303;
    }
}


/** @fn void sci2LowLevelInterrupt(void)
*   @brief Low Level Interrupt for SCI2
*/
#if 1
    #pragma INTERRUPT(sci2LowLevelInterrupt)
#endif

void sci2LowLevelInterrupt(void)
{
    unsigned vec = sciREG2->INTVECT1;

    switch (vec)
    {
    case 1:
        sciNotification(sciREG2, SCI_WAKE_INT);
        break;
    case 3:
        sciNotification(sciREG2, SCI_PE_INT);
        break;
    case 6:
        sciNotification(sciREG2, SCI_FE_INT);
        break;
    case 7:
        sciNotification(sciREG2, SCI_BREAK_INT);
        break;
    case 9:
        sciNotification(sciREG2, SCI_OE_INT);
        break;

    case 11:
        /* receive */
        {   unsigned byte = sciREG2->RD;

            if (g_sciTransfer[1].length > 0)
            {
                *g_sciTransfer[1].data++ = byte;
                g_sciTransfer[1].length--;
                if (g_sciTransfer[1].length == 0)
                {
                    sciNotification(sciREG2, SCI_RX_INT);
                }
            }
        }
        break;

    case 12:
        /* transmit */
        if (--g_sciTransfer[1].length > 0)
        {
            sciREG2->TD = *g_sciTransfer[0].data++;
        }
        else
        {
            sciREG2->CLRINT = SCI_TX_INT;
            sciNotification(sciREG2, SCI_TX_INT);
        }
        break;

    default:
        /* phantom interrupt, clear flags and return */
        sciREG2->FLR = sciREG2->SETINTLVL & 0x07000303;
    }
}

#endif
