/*----------------------------------------------------------------------------*/
/* sys_startup.c                                        03/15/11              */
/*                                                                            */
/* (c) Texas Instruments 2011, All rights reserved.                           */
/*                                                                            */
/* Versions:                                                                  */
/* HALCOGEN         */
/* TMS470R3x        */
/* TICGEN           */
/*                                                                            */
/*----------------------------------------------------------------------------*/

/* Include Files */

#include "system.h"
#include "sys_vim.h"

typedef void (*handler_fptr)(const unsigned char *in, unsigned char *out);

#pragma WEAK(__TI_Handler_Table_Base)
#pragma WEAK(__TI_Handler_Table_Limit)
#pragma WEAK(__TI_CINIT_Base)
#pragma WEAK(__TI_CINIT_Limit)

extern unsigned int __TI_Handler_Table_Base;
extern unsigned int __TI_Handler_Table_Limit;
extern unsigned int __TI_CINIT_Base;
extern unsigned int __TI_CINIT_Limit;
extern unsigned *   __binit__;

#define NVIC  0xE000E000

/*------------------------------------------------------------------------------*/
/* C auto-initialization 							*/
void _init()
{
/* initalise copy table */
    if ((unsigned *)&__binit__ != (unsigned *)0xFFFFFFFF)
    {
        extern void copy_in(void *binit);
        copy_in((void *)&__binit__);
    }

    /* initalise the C global variables */
    if (&__TI_Handler_Table_Base < &__TI_Handler_Table_Limit)
    {
        unsigned char **tablePtr   = (unsigned char **)&__TI_CINIT_Base;
        unsigned char **tableLimit = (unsigned char **)&__TI_CINIT_Limit;

        while (tablePtr < tableLimit)
        {
            unsigned char *loadAdr = *tablePtr++;
            unsigned char *runAdr  = *tablePtr++;
            unsigned char  idx     = *loadAdr++;
            handler_fptr   handler = (handler_fptr)(&__TI_Handler_Table_Base)[idx];

            (*handler)((const unsigned char *)loadAdr, runAdr);
        }
    }
}



/*------------------------------------------------------------------------------*/
/* Enable interrupt Handling in NVIC 		                             	*/
void NVIC_enable()
{
	unsigned int *SETINT0;	/* Irq 0 to 31 Set Enable Register	     	*/
	unsigned int *PRIOR0;	/* Irq 0 to 3 Priority Register			*/
	unsigned int *PRIOR1;   /* Irq 4 to 7 Priority Register			*/
	unsigned int *SYSHDLR;  /* System Handler Control and State Register 	*/


	SETINT0 	= (unsigned int *)(NVIC + 0x100);
	PRIOR0 		= (unsigned int *)(NVIC + 0x400);
	PRIOR1 		= (unsigned int *)(NVIC + 0x404);
	SYSHDLR         = (unsigned int *)(NVIC + 0xD24);

	*PRIOR0 	= 0x80a0c0e0;
	*PRIOR1 	= 0x00204060;
	*SYSHDLR        = 0x00070000; 
	*SETINT0 	= 0x000000FF;
	
#if VIM_CONFIG	
	vimCfg();	/* Configure M3VIM Channels properties */
#endif

#if (VIM_ENABLE_NESTED_INTERRUPTS  == FALSE)
	nestEnable();
#endif 


}



/*------------------------------------------------------------------------------*/
/* Set ECP_CLK Pin  		                             	                */

void _set_ecp_clk()
{
  systemREG->SYSPC1  = 0x1U;                  /* ECP Pin as Functional */

  if(SYS_PLL == SYS_VCLK)
  {
     *(unsigned *)0xFFFFFFD4 = 0x00800000;
  }
  else
  {	  
     systemREG->CLKTEST = (0x5U << 16U) | SYS_PLL;
  }
}
								
/*- End of sys_startup.c--------------------------------------------------------*/



